# TPK4186 - Advanced Tools for Performance Engineering
# HTMLInterface.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# This work is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required modules
# 2. HTML Serializer
# 3. HTML Creator

# 1. Required modules
# -------------------

import re
from Retailer import *


# 2. HTML Serializer
# ------------------

class HTMLSerializer:
  def __init__(self):
    self.indentation = "\t"
    self.depth = 0

  def SerializeListOfProducts(self, retailer):
    result = self.SerializeOpenTag("ul")
    self.depth += 1
    for product in retailer.GetProducts():
      result += self.SerializeOpenTag("li")
      self.depth += 1
      result += self.SerializeProductDescription(product)
      self.depth -= 1
      result += self.SerializeCloseTag("li")
    self.depth -= 1
    result += self.SerializeCloseTag("ul")
    return result

  def SerializeProductDescription(self, product):
    return product.GetName() + " (profit per sale: " + self.SerializeCurrency(product.GetProfitPerSale()) + ")\n"

  def SerializeTableOfSales(self, retailer):
    result = self.SerializeOpenTag("table")
    self.depth += 1
    # Header
    result += self.SerializeOpenTag("tr")
    self.depth += 1
    result += self.SerializeOpenTag("th")
    self.depth += 1
    result += "Product\n"
    self.depth -= 1
    result += self.SerializeCloseTag("th")
    for monthNumber in range(1, 13):
      result += self.SerializeOpenTag("th")
      self.depth += 1
      result += self.GetMonthTrigramFromMonthNumber(monthNumber) + "\n"
      self.depth -= 1
      result += self.SerializeCloseTag("th")
    self.depth -= 1
    result += self.SerializeCloseTag("tr")
    # Sales
    for product in retailer.GetProducts():
      result += self.SerializeOpenTag("tr")
      self.depth += 1
      result += self.SerializeOpenTag("td")
      self.depth += 1
      result += product.GetName() + "\n"
      self.depth -= 1
      result += self.SerializeCloseTag("td")
      for monthNumber in range(1, 13):
        result += self.SerializeOpenTag("td")
        self.depth += 1
        result += str(product.GetMonthlySales(monthNumber)) + "\n"
        self.depth -= 1
        result += self.SerializeCloseTag("td")
      self.depth -= 1
      result += self.SerializeCloseTag("tr")
    self.depth -= 1
    result += self.SerializeCloseTag("table")
    return result

  def SerializeTableOfProfits(self, retailer):
    result = self.SerializeOpenTag("table")
    self.depth += 1
    # Header
    result += self.SerializeOpenTag("tr")
    self.depth += 1
    result += self.SerializeOpenTag("th")
    self.depth += 1
    result += "Product\n"
    self.depth -= 1
    result += self.SerializeCloseTag("th")
    for monthNumber in range(1, 13):
      result += self.SerializeOpenTag("th")
      self.depth += 1
      result += self.GetMonthTrigramFromMonthNumber(monthNumber) + "\n"
      self.depth -= 1
      result += self.SerializeCloseTag("th")
    result += self.SerializeOpenTag("th")
    self.depth += 1
    result += "Year\n"
    self.depth -= 1
    result += self.SerializeCloseTag("th")
    self.depth -= 1
    result += self.SerializeCloseTag("tr")
    # Profits
    for product in retailer.GetProducts():
      result += self.SerializeOpenTag("tr")
      self.depth += 1
      result += self.SerializeOpenTag("td")
      self.depth += 1
      result += product.GetName() + "\n"
      self.depth -= 1
      result += self.SerializeCloseTag("td")
      for monthNumber in range(1, 13):
        result += self.SerializeOpenTag("td")
        self.depth += 1
        result += self.SerializeCurrency(product.ComputeMonthlyProfit(monthNumber)) + "\n"
        self.depth -= 1
        result += self.SerializeCloseTag("td")
      result += self.SerializeOpenTag("td")
      self.depth += 1
      result += self.SerializeCurrency(product.ComputeYearlyProfit()) + "\n"
      self.depth -= 1
      result += self.SerializeCloseTag("td")
      self.depth -= 1
      result += self.SerializeCloseTag("tr")
    # Total
    result += self.SerializeOpenTag("tr")
    self.depth += 1
    result += self.SerializeOpenTag("td")
    self.depth += 1
    result += "Total\n"
    self.depth -= 1
    result += self.SerializeCloseTag("td")
    for monthNumber in range(1, 13):
      result += self.SerializeOpenTag("td")
      self.depth += 1
      result += self.SerializeCurrency(retailer.ComputeMonthlyProfit(monthNumber))
      self.depth -= 1
      result += self.SerializeCloseTag("td")
    result += self.SerializeOpenTag("td")
    self.depth += 1
    result += self.SerializeCurrency(retailer.ComputeYearlyProfit())
    self.depth -= 1
    result += self.SerializeCloseTag("td")
    self.depth -= 1
    result += self.SerializeCloseTag("tr")
    self.depth -= 1
    result += self.SerializeCloseTag("table")
    return result

  def SerializeIndentation(self):
    result = ""
    for de in range(0, self.depth):
      result += self.indentation
    return result

  def SerializeOpenTag(self, name):
    return self.SerializeIndentation() + "<" + name + ">\n"

  def SerializeCloseTag(self, name):
    return self.SerializeIndentation() + "</" + name + ">\n"

  def SerializeCurrency(self, value):
   return "{0:.2f}".format(value)

  def GetMonthTrigramFromMonthNumber(self, number):
    if number==1:
      return "Jan"
    if number==2:
      return "Feb"
    if number==3:
      return "Mar"
    if number==4:
      return "Apr"
    if number==5:
      return "May"
    if number==6:
      return "Jun"
    if number==7:
      return "Jul"
    if number==8:
      return "Aug"
    if number==9:
      return "Sep"
    if number==10:
      return "Oct"
    if number==11:
      return "Nov"
    if number==12:
      return "Dec"
    return None

# 3. HTML Creator
# --------------

class HTMLCreator:
  def __init__(self):
    self.serializer = HTMLSerializer()

  def ExportRetailerSalesAtHTMLFormat(self, retailer, templateFileName, targetFileName):
    try:
      templateFile = open(templateFileName, "r")
    except:
      sys.stderr.write('Unable to open file "%s"\n' % templateFileName)
      sys.stderr.flush()
      return 1
    try:
      targetFile = open(targetFileName, "w")
    except:
      sys.stderr.write('Unable to open file "%s"\n' % targetFileName)
      sys.stderr.flush()
      templateFile.close()
      return 1
    self.PrintReport(retailer, templateFile, targetFile)
    templateFile.close()
    targetFile.flush()
    targetFile.close()
    return 0

  def PrintReport(self, retailer, templateFile, targetFile):
    for line in templateFile:
      line = line.rstrip()
      if re.search(r'__PRODUCTS__', line):
        line = re.sub(r'__PRODUCTS__', self.serializer.SerializeListOfProducts(retailer), line)
      elif re.search(r'__SALES__', line):
        line = re.sub(r'__SALES__', self.serializer.SerializeTableOfSales(retailer), line)
      elif re.search(r'__PROFITS__', line):
        line = re.sub(r'__PROFITS__', self.serializer.SerializeTableOfProfits(retailer), line)
      targetFile.write(line + "\n")

