# TPK4186 - Advanced Tools for Performance Engineering
# TSVInterface.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# This work is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required modules
# 2. TSV Printer
# 3. TSV Parser

# 1. Required modules
# -------------------

from Retailer import *

# 2. TSV Printer
# --------------

class TSVPrinter:
  def __init__(self):
    self.separator = "\t"

  def ExportRetailerSalesAtTSVFormat(self, retailer, fileName):
    try:
      output = open(fileName, "w")
    except:
      sys.stderr.write('Unable to open file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    self.PrintRetailerSalesAtTSVFormat(retailer, output)
    output.close()
    return 0

  def PrintRetailerSalesAtTSVFormat(self, retailer, output):
    output.write("Product" + self.separator)
    output.write("Profit" + self.separator)
    output.write("Jan" + self.separator)
    output.write("Feb" + self.separator)
    output.write("Mar" + self.separator)
    output.write("Apr" + self.separator)
    output.write("May" + self.separator)
    output.write("Jun" + self.separator)
    output.write("Jul" + self.separator)
    output.write("Aug" + self.separator)
    output.write("Sep" + self.separator)
    output.write("Oct" + self.separator)
    output.write("Nov" + self.separator)
    output.write("Dec" + "\n")
    for product in retailer.GetProducts():
      self.PrintProductSalesAtTSVFormat(product, output)
    output.flush()

  def PrintProductSalesAtTSVFormat(self, product, output):
    output.write(product.GetName() + self.separator)
    self.PrintCurrency(product.GetProfitPerSale(), output)
    for monthNumber in range(1, 13):
      output.write(self.separator + str(product.GetMonthlySales(monthNumber)))
    output.write("\n")

  def PrintCurrency(self, value, output):
    output.write("{0:.2f}".format(value))

  def ExportRetailerProfitAtTSVFormat(self, retailer, fileName):
    try:
      output = open(fileName, "w")
    except:
      sys.stderr.write('Unable to open file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    self.PrintRetailerProfitAtTSVFormat(retailer, output)
    output.close()
    return 0

  def PrintRetailerProfitAtTSVFormat(self, retailer, output):
    output.write("Product" + self.separator)
    output.write("Jan" + self.separator)
    output.write("Feb" + self.separator)
    output.write("Mar" + self.separator)
    output.write("Apr" + self.separator)
    output.write("May" + self.separator)
    output.write("Jun" + self.separator)
    output.write("Jul" + self.separator)
    output.write("Aug" + self.separator)
    output.write("Sep" + self.separator)
    output.write("Oct" + self.separator)
    output.write("Nov" + self.separator)
    output.write("Dec" + self.separator)
    output.write("Year" + "\n")
    for product in retailer.GetProducts():
      self.PrintProductProfitAtTSVFormat(product, output)
    output.write("Total")
    for monthNumber in range(1, 13):
      output.write(self.separator)
      self.PrintCurrency(retailer.ComputeMonthlyProfit(monthNumber), output)
    output.write(self.separator)
    self.PrintCurrency(retailer.ComputeYearlyProfit(), output)
    output.write("\n")
    output.flush()

  def PrintProductProfitAtTSVFormat(self, product, output):
    output.write(product.GetName())
    for monthNumber in range(1, 13):
      output.write(self.separator)
      self.PrintCurrency(product.ComputeMonthlyProfit(monthNumber), output)
    output.write(self.separator)
    self.PrintCurrency(product.ComputeYearlyProfit(), output)
    output.write("\n")

# 3. TSV Parser
# -------------

class TSVParser:
  def __init__(self):
    self.separator = "\t"

  def ImportRetailerSalesAtTSVFormat(self, retailer, fileName):
    try:
      input = open(fileName, "r")
    except:
      sys.stderr.write('Unable to open file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    error = self.ParseRetailerSalesAtTSVFormat(retailer, input)
    input.close()
    return error

  def ParseRetailerSalesAtTSVFormat(self, retailer, input):
    lineNumber = 0
    for line in input:
      lineNumber += 1
      if lineNumber==1: # Header
        continue
      line.rstrip()
      tokens = line.split(self.separator)
      if len(tokens)<14:
        sys.stderr.write('Not enough data line %d\n' % lineNumber)
        sys.stderr.flush()
        return 1
      productName = tokens[0]
      product = retailer.NewProduct(productName)
      if self.IsFloat(tokens[1]):
        profitPerSale = float(tokens[1])
        product.SetProfitPerSale(profitPerSale)
      else:
        sys.stderr.write('Line %d: "%s" is not a correct profit per sale"\n' % (lineNumber, tokens[1]))
        sys.stderr.flush()
        return 1
      for monthNumber in range(1, 13):
        if self.IsInteger(tokens[monthNumber+1]):
          quantity = int(tokens[monthNumber+1])
          product.SetMonthlySales(monthNumber, quantity)
        else:
          sys.stderr.write('Line %d: "%s" is not a monthly quantity"\n' % (lineNumber, tokens[monthNumber+1]))
          sys.stderr.flush()
          return 1
    return 0

  def IsInteger(self, token):
    return re.match("[0-9]+", token)

  def IsFloat(self, token):
    return re.match("[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?", token)


